/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
KEYBOARD_DATA kbdData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************


/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

#define MAX_KEYCODE 16
uint8_t kbdBuffer[MAX_KEYCODE];
uint8_t kbdSize = 0;
uint8_t kbdReadPt = 0;
uint8_t kbdWritePt = 0;


void kbdBufferClear()
{
    kbdSize = 0;
    kbdReadPt = 0;
    kbdWritePt = 0;
}

void kbdBufferWrite(uint8_t keyCode)
{
    if(MAX_KEYCODE - kbdSize < 1 )
    {
        return;
    }
    kbdBuffer[kbdWritePt] = keyCode;
    kbdSize++;
    kbdWritePt++;
    if(kbdWritePt >= MAX_KEYCODE)
    {
        kbdWritePt = 0;
    }
    
}

uint8_t kbdBufferRead()
{
    uint8_t keyCode;

    if(kbdSize == 0)
    {
        return 0;
    }
    keyCode = kbdBuffer[kbdReadPt];
    kbdSize--;
    kbdReadPt++;
    if(kbdReadPt >= MAX_KEYCODE)
    {
        kbdReadPt = 0;
    }
    return keyCode;
}


void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;


    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
    kbdBufferClear();
//    PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5, 1);
}

/*******************************************************
 * USB HOST Layer Events - Host Event Handler
 *******************************************************/

USB_HOST_EVENT_RESPONSE APP_USBHostEventHandler (USB_HOST_EVENT event, void * eventData, uintptr_t context)
{
    switch(event)
    {
        case USB_HOST_EVENT_DEVICE_UNSUPPORTED:
            break;
        default:
            break;
    }
    return USB_HOST_EVENT_RESPONSE_NONE;
}



/*******************************************************
 * USB HOST HID Layer Events - Application Event Handler
 *******************************************************/
void APP_USBHostHIDKeyboardEventHandler(USB_HOST_HID_KEYBOARD_HANDLE handle, 
        USB_HOST_HID_KEYBOARD_EVENT event, void * pData)
{   
    switch ( event)
    {
        case USB_HOST_HID_KEYBOARD_EVENT_ATTACH:
            kbdData.keyboard_handle = handle;
            kbdData.state =  KEYBOARD_STATE_DEVICE_ATTACHED;
//            memset(&kbdData.lastData, 0, sizeof(kbdData.lastData));
            kbdData.prevKeyCode = 0;

            kbdData.capsLockPressed = false;
            kbdData.scrollLockPressed = false;
            kbdData.numLockPressed = false;
            kbdData.outputReport = 0;
            break;

        case USB_HOST_HID_KEYBOARD_EVENT_DETACH:
            kbdData.keyboard_handle = handle;
            kbdData.state = KEYBOARD_STATE_DEVICE_DETACHED;
//            memset(&kbdData.lastData, 0, sizeof(kbdData.lastData));
            kbdData.prevKeyCode = 0;
            kbdData.capsLockPressed = false;
            kbdData.scrollLockPressed = false;
            kbdData.numLockPressed = false;
            kbdData.outputReport = 0;
            break;

        case USB_HOST_HID_KEYBOARD_EVENT_REPORT_RECEIVED:
            kbdData.keyboard_handle = handle;
            kbdData.state = KEYBOARD_STATE_READ_HID;
            /* Keyboard Data from device */
            memcpy(&kbdData.data, pData, sizeof(kbdData.data));
            break;

        default:
            break;
    }
 //   PLIB_PORTS_PinToggle(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5);
 //   USB_HOST_HID_KEYBOARD_ReportSend(kbdData.keyboard_handle, 0x01);
   return;
}


uint8_t getAlfabet(uint8_t keyCode)
{
    uint8_t decodedKeyCode = 0;    
    if(kbdData.prevKeyCode != keyCode && keyCode != 0x00)
    {
        if(keyCode >= USB_HID_KEYBOARD_KEYPAD_KEYBOARD_A &&
            keyCode <= USB_HID_KEYBOARD_KEYPAD_KEYBOARD_Z)
        {
            decodedKeyCode = keyCode - USB_HID_KEYBOARD_KEYPAD_KEYBOARD_A + 'a';
            if(kbdData.data.modifierKeysData.leftShift == 1 || kbdData.data.modifierKeysData.rightShift == 1)
            {
                decodedKeyCode -= 0x20;
            }
        }
    }
    kbdData.prevKeyCode = keyCode;
    return decodedKeyCode;
    
}

 void decodeKey()
 {
    uint8_t count, keyCode;
    
    /* We need to display only the non modifier keys */
    for(count = 0; count < 6; count++)
    {
        if(kbdData.data.nonModifierKeysData[count].event == USB_HID_KEY_PRESSED)
        {
            keyCode = getAlfabet(kbdData.data.nonModifierKeysData[count].keyCode);
            if(keyCode != 0x00)
            {
                kbdBufferWrite(keyCode);
            }
        }
    }
     
 }


void APP_Keyboard_Tasks ( void )
{
    switch ( kbdData.state )
    {
        case KEYBOARD_STATE_WAIT_FOR_DEVICE_ATTACH:
            break;

        case KEYBOARD_STATE_DEVICE_ATTACHED:
            
            kbdData.state = KEYBOARD_STATE_READ_HID;
            break;

        case KEYBOARD_STATE_READ_HID:
            decodeKey();
                
            break;
        case KEYBOARD_STATE_DEVICE_DETACHED:
            kbdData.state = KEYBOARD_STATE_WAIT_FOR_DEVICE_ATTACH;
            break;

        default:
            break;
    }
}

/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
            USB_HOST_EventHandlerSet(APP_USBHostEventHandler, 0);
            USB_HOST_HID_KEYBOARD_EventHandlerSet(APP_USBHostHIDKeyboardEventHandler);
            USB_HOST_BusEnable(0);
        
            if (appInitialized)
            {
            
                appData.state = APP_STATE_WAIT_BUS;
            }
            break;
        }
        case APP_STATE_WAIT_BUS:
        {
            if(USB_HOST_BusIsEnabled(0))
            {
                /* This means host operation is enabled. We can
                 * move on to the next state */
                appData.state = APP_STATE_SERVICE_TASKS;
            }
        }

        case APP_STATE_SERVICE_TASKS:
        {
            APP_Keyboard_Tasks();
            break;
        }


        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

 

/*******************************************************************************
 End of File
 */
